import 'package:parse_server_sdk_flutter/parse_server_sdk.dart';

class MealPortionType {
  late String? objectId;
  late String value;
  late String? notes;

  MealPortionType(ParseObject? object) {
    if (object != null) {
      objectId = object.get<String>('objectId');
      value = object.get<String>('value')!;
      notes = object.get<String>('notes');
    }
  }

  static Future<List<MealPortionType>> fetchAll() async {
    QueryBuilder<ParseObject> query =
        QueryBuilder<ParseObject>(ParseObject('MealPortionType'));
    final ParseResponse apiResponse = await query.query();

    if (apiResponse.success && apiResponse.results != null) {
      // return apiResponse.results as List<ParseObject>;
      return apiResponse.results!.map((e) => MealPortionType(e as ParseObject)).toList();
    } else {
      return [];
    }
  }

  static Future<MealPortionType?> get(String objectId) async {
    QueryBuilder<ParseObject> query =
        QueryBuilder<ParseObject>(ParseObject('MealPortionType'))
          ..whereEqualTo('objectId', objectId);
    final ParseResponse apiResponse = await query.query();

    if (apiResponse.success && apiResponse.results != null) {
      return MealPortionType(apiResponse.result.first);
    }
  }

  static Future<void> save({
    required String value,
    String? notes,
  }) async {
    final mealPortionType = ParseObject('MealPortionType')
      ..set('value', value)
      ..set('notes', notes);
    await mealPortionType.save();
  }

  static Future<void> update(
      String objectId, {
        String? value,
        String? notes,
      }) async {
    var mealPortionType = ParseObject('MealPortionType')..objectId = objectId;
    if (value != null) {
      mealPortionType.set('value', value);
    }
    if (notes != null) {
      mealPortionType.set('notes', notes);
    }
    await mealPortionType.save();
  }

  Future<void> delete() async {
    var mealPortionType = ParseObject('MealPortionType')..objectId = objectId;
    await mealPortionType.delete();
  }
}
